/**
 * 网络请求
 */
import ext from './extend.js';
import tok from './token.js';

/**
 * @typedef {Object} RequestResult
 * @property {boolean} success - 请求是否成功标记
 * @property {string} message - 请求失败消息
 * @property {Object} data - 请求成功接收的数据
 */

export default {
    /**
     * 设置基础地址
     * @param {string} url - 基础地址
     */
    setBaseUrl(url) {
        if (!ext.isString(url)) {
            url = '';
        }
        ext.addStorage('base_url', url);
    },

    /**
     * 获取基础地址
     */
    getBaseUrl() {
        return ext.readStorage('base_url');
    },

    /**
     * 网络请求
     * @param {Object} opts - 请求参数
     * @param {string} opts.url - 请求地址
     * @param {string} opts.method - 请求方法，默认 get
     * @param {Object} opts.data - 请求参数
     * @param {Object} opts.header - 请求头
     * @returns {RequestResult} 网络请求结果对象
     */
    request(opts) {
        let _this = this;
        return new Promise(async (resolve, reject) => {
            if (!ext.isObject(opts)) {
                resolve({
                    success: false,
                    message: '参数不能为空'
                });
                return;
            }
            let url = '';
            let baseUrl = _this.getBaseUrl();
            if (!ext.isUrl(opts.url)) {
                if (ext.isEmptyString(baseUrl)) {
                    resolve({
                        success: false,
                        message: '未设置 BaseUrl'
                    });
                    return;
                } else {
                    url = baseUrl + opts.url;
                }
            }
            uni.request({
                url: url,
                method: opts.method || 'get',
                dataType: opts.dataType || 'json',
                responseType: opts.responseType || 'json',
                header: opts.header || {},
                data: opts.data,
                success(res) {
                    resolve({
                        success: true,
                        data: res
                    });
                },
                fail(ex) {
                    ex = ext.errorFormatter(ex);
                    resolve({
                        success: false,
                        msg: ex.message
                    });
                }
            });

        });
    },

    /**
     * Token 网络请求
     * 在 header 中带 Authorization 参数。
     * @param {Object} opts - 请求参数
     * @param {string} opts.url - 请求地址
     * @param {string} opts.method - 请求方法，默认 get
     * @param {Object} opts.data - 请求参数
     * @param {Object} opts.header - 请求头
     * @returns {RequestResult} 网络请求结果对象
     */
    async tokenRequest(opts) {
        if (!ext.isObject(opts)) {
            return {
                success: false,
                message: '参数不能为空'
            };
        }
        let token = tok.get();
        if (ext.isNonEmptyString(token)) {
            opts.header = {
                Authorization: token
            };
        }
        return await this.request(opts);
    }
};